@@ -20,7 +20,6 @@ _log = getLogger(__name__)
 
 MAX_WORKERS_KEY = 'max_workers'
 PARENT_CALLS_KEY = 'parent_calls_tracked'
-KILL_TIMEOUT = 3  # seconds
 
 NAMEKO_CONTEXT_KEYS = (
     'language',
@@ -133,6 +132,7 @@ class WorkerContext(WorkerContextBase):
 class ManagedThreadContainer(object):
     def __init__(self):
         self._active_threads = set()
+        self._protected_threads = set()
         self._being_killed = False
         self._died = Event()
 
@@ -142,8 +142,8 @@ class ManagedThreadContainer(object):
         `_handle_container_stop` is called to give sub-classes an opportunity
         to clean up dependencies.
 
-        At this point there should be no more active threads. In case there
-        are any active threads, they are killed by the container.
+        At this point there should be no more managed threads. In case there
+        are any managed threads, they are killed by the container.
         """
         if self._died.ready():
             _log.debug('already stopped %s', self)
@@ -154,16 +154,19 @@ class ManagedThreadContainer(object):
         with log_time(_log.debug, 'stopped %s in %0.3f sec', self):
             self._handle_container_stop()
 
-            # just in case there was a provider not taking care of it's worker
+            # just in case there was a provider not taking care of its workers,
+            # or a dependency not taking care of its protected threads
             self._kill_active_threads()
+            self._kill_protected_threads()
 
             self._died.send(None)
 
     def kill(self, exc):
         """ Kill the container in a semi-graceful way.
 
-        `_handle_container_kill` is called to give sub-classes an opportunity
-        to clean up dependencies. Once complete, any remaining active threads
+        All non-protected managed threads are killed first. This includes
+        all active workers generated by :meth:`ServiceContainer.spawn_worker`.
+        Next, dependencies are killed. Finally, any remaining protected threads
         are killed.
 
         The container dies with the given ``exc``.
@@ -183,9 +186,11 @@ class ManagedThreadContainer(object):
 
         _log.info('killing %s due to "%s"', self, exc)
 
-        self._handle_container_kill(exc)
-
+        self._kill_entrypoints(exc)
         self._kill_active_threads()
+        self._handle_container_kill(exc)
+        self._kill_protected_threads()
+
         self._died.send_exception(exc)
 
     def wait(self):
@@ -199,51 +204,80 @@ class ManagedThreadContainer(object):
         """
         return self._died.wait()
 
-    def spawn_managed_thread(self, run_method):
-        """ Spawn a lifecycle-managed thread, which calls the ``run_method``
-        once it has been started.
+    def spawn_managed_thread(self, run_method, protected=False):
+        """ Spawn a managed thread to run ``run_method``.
 
-        Any errors raised inside the ``run_method`` cause the container to be
+        Threads can be marked as ``protected``, which means the container will
+        not forcibly kill them until after all dependencies have been killed.
+        Dependencies that require a managed thread to complete their kill
+        procedure should ensure to mark them as ``protected``.
+
+        Any uncaught errors inside ``run_method`` cause the container to be
         killed.
 
-        It is the entrypoint provider's responsibility to react to ``stop()``
-        calls and terminate it's spawned threads.
-
-        Threads are killed automatically by the container if they are
-        still running after all their providers have been stopped.
+        It is the caller's responsibility to terminate their spawned threads.
+        Threads are killed automatically if they are still running after
+        all dependencies are stopped during :meth:`ServiceContainer.stop`.
 
         Entrypoints may only create separate threads using this method,
         to ensure they are life-cycle managed.
         """
         gt = eventlet.spawn(run_method)
-        self._active_threads.add(gt)
+        if not protected:
+            self._active_threads.add(gt)
+        else:
+            self._protected_threads.add(gt)
         gt.link(self._handle_thread_exited)
         return gt
 
     def _handle_container_stop(self):
+        """ Called when a container is gracefully stopped.
         """
-        Called when a container is stopped, this is an opportunity to
-        gracefully stop any dependencies a subclass may accrue.
+        return
+
+    def _kill_entrypoints(self, exc):
+        """ Called when a container is killed.
         """
         return
 
     def _handle_container_kill(self, exc):
-        """
-        Called when a container is killed, this is an opportunity to kill any
-        dependencies a subclass may accrue
+        """ Called when a container is killed.
         """
         return
 
     def _kill_active_threads(self):
+        """ Kill all managed threads that were not marked as "protected" when
+        they were spawned.
+
+        This set will include all worker threads generated by
+        :meth:`ServiceContainer.spawn_worker`.
+
+        See :meth:`ManagedThreadContainer.spawn_managed_thread`
+        """
         num_active_threads = len(self._active_threads)
 
         if num_active_threads:
-            _log.warning('killing (%s) active threads', num_active_threads)
+            _log.warning('killing %s active thread(s)', num_active_threads)
             for gt in list(self._active_threads):
                 gt.kill()
 
+    def _kill_protected_threads(self):
+        """ Kill any managed threads marked as protected when they were
+        spawned.
+
+        See :meth:`ManagedThreadContainer.spawn_managed_thread`
+        """
+        num_protected_threads = len(self._protected_threads)
+
+        if num_protected_threads:
+            _log.warning('killing %s protected thread(s)',
+                         num_protected_threads)
+            for gt in list(self._protected_threads):
+                gt.kill()
+
     def _handle_thread_exited(self, gt):
-        self._active_threads.remove(gt)
+        self._active_threads.discard(gt)
+        self._protected_threads.discard(gt)
 
         try:
             gt.wait()
@@ -348,23 +382,11 @@ class ServiceContainer(ManagedThreadContainer):
         # finally, stop nested dependencies
         dependencies.nested.all.stop()
 
+    def _kill_entrypoints(self, exc):
+        self.dependencies.entrypoints.all.kill(exc)
+
     def _handle_container_kill(self, exc):
-        """ Kill the container in a semi-graceful way.
-
-        First all dependencies have a chance to kill themselves
-        within a given time limit (``KILL_TIMEOUT``).
-
-        To do so they must implement a ``kill(exc)`` method and take care
-        of shutting down any resources when that method is called on them.
-        """
-        self._kill_dependencies(exc)
-
-    def _kill_dependencies(self, exc):
-        try:
-            with eventlet.Timeout(KILL_TIMEOUT):
-                self.dependencies.all.kill(exc)
-        except eventlet.Timeout:
-            _log.warning('timeout waiting for dependencies.kill %s', self)
+        self.dependencies.all.kill(exc)
 
     def _run_worker(self, worker_ctx, handle_result):
         _log.debug('setting up %s', worker_ctx,
